"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransformModule = exports.IGNORE_PROPS = void 0;
const utils_1 = require("../utils");
exports.IGNORE_PROPS = new Set([
    'ref',
    'refInFor',
    'slot',
    'props',
    'attrs',
    'on',
    'nativeOn',
    'directives',
    'scopedSlots',
    'key',
    'class',
    'style',
    'domProps',
    // vue3
    'ref_for',
]);
class TransformModule {
    constructor(options) {
        Object.defineProperty(this, "matcher", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "preTransformNode", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: el => {
                const { tag } = el;
                const isCustomElement = this.matcher.isCustomElement(tag);
                if (!isCustomElement) {
                    return el;
                }
                return this._transformVModel(this._transformDotBinging(el));
            }
        });
        /**
         * transform an AST node after built-ins like v-if, v-for are processed
         */
        Object.defineProperty(this, "postTransformNode", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: el => {
                this._transformDomProps(el);
            }
        });
        /**
         * 支持 .bind 语法
         */
        Object.defineProperty(this, "_transformDotBinging", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: (el) => {
                const { attrsList, attrsMap } = el;
                // 支持 .bind 语法
                if (attrsList == null || attrsList.length === 0) {
                    return el;
                }
                for (const attr of attrsList) {
                    const name = attr.name;
                    let newName = name;
                    if (!name.startsWith('.')) {
                        continue;
                    }
                    // 替换 . 为 :
                    newName = ':' + newName.slice(1);
                    // 追加 .prop 修饰符
                    newName = newName.replace('.prop', '');
                    newName += '.prop';
                    // 修改 attrsMap
                    const value = attrsMap[name];
                    // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
                    delete attrsMap[name];
                    attrsMap[newName] = value;
                    attr.name = newName;
                }
                return el;
            }
        });
        this.matcher = (0, utils_1.createMatcher)(options);
    }
    genData() {
        return '';
    }
    transformNode(el) {
        return el;
    }
    /**
     * 支持 v-model 语法 preTransformNode
     */
    _transformVModel(el) {
        const { attrsList, attrsMap } = el;
        if (attrsList == null || attrsList.length === 0) {
            return el;
        }
        const vModel = attrsList.find(attr => attr.name.startsWith('v-model'));
        if (vModel == null) {
            return el;
        }
        const lazy = vModel.name.includes('.lazy');
        const eventName = lazy ? 'change' : 'input';
        const valueBinding = { name: ':value', value: vModel.value };
        const eventHandler = { name: `@${eventName}`, value: `(${vModel.value}) = $event.detail[0]` };
        // 删除 v-model
        attrsList.splice(attrsList.indexOf(vModel), 1, valueBinding, eventHandler);
        // eslint-disable-next-line @typescript-eslint/no-dynamic-delete
        delete attrsMap[vModel.name];
        attrsMap[valueBinding.name] = valueBinding.value;
        attrsMap[eventHandler.name] = eventHandler.value;
        return el;
    }
    /**
     * 强制使用 domProps 传入
     * TODO: 通过干预 mustUseProp 实现
     * @param el
     * @returns
     */
    _transformDomProps(el) {
        const { tag, attrs, dynamicAttrs, directives } = el;
        const isCustomElement = this.matcher.isCustomElement(tag);
        if (!isCustomElement || (!this._hasAttribute(attrs) && !this._hasAttribute(dynamicAttrs))) {
            return;
        }
        const mustUseProp = this.matcher.mustUseProp(tag);
        /**
         * 迁移属性
         */
        const moveAttributes = (attributes) => {
            const clone = [...attributes];
            for (const attr of clone) {
                const ignore = this._defaultAttributeIgnore(attr.name);
                if (ignore) {
                    continue;
                }
                // 纯文本不转换
                if (typeof attr.value === 'string' && attr.value.startsWith('"')) {
                    continue;
                }
                // 放入 props 中
                const idx = attributes.indexOf(attr);
                if (idx !== -1) {
                    attributes.splice(idx, 1);
                }
                if (el.props == null) {
                    el.props = [];
                }
                el.props.push(attr);
            }
        };
        // 没有强制使用 dom 属性传入
        if (!mustUseProp) {
            return;
        }
        if (attrs) {
            moveAttributes(attrs);
        }
        if (dynamicAttrs) {
            moveAttributes(dynamicAttrs);
        }
        // 处理 v-bind="props" 透传语法
        if (directives === null || directives === void 0 ? void 0 : directives.length) {
            directives.forEach(d => {
                if (d.name === 'bind') {
                    // 默认使用 prop 修饰
                    if (d.modifiers == null) {
                        d.modifiers = { prop: true };
                    }
                    else {
                        d.modifiers.prop = true;
                    }
                }
            });
        }
    }
    /**
     * 默认忽略的属性
     * @param {string} attr
     */
    _defaultAttributeIgnore(attr) {
        return exports.IGNORE_PROPS.has(attr);
    }
    _hasAttribute(attrs) {
        return attrs != null && attrs.length > 0;
    }
}
exports.TransformModule = TransformModule;
